@@ -28,15 +28,19 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
+#include <string.h>
 
 #include "gtkactiongroup.h"
+#include "gtkbuildable.h"
 #include "gtkiconfactory.h"
+#include "gtkicontheme.h"
 #include "gtkstock.h"
 #include "gtktoggleaction.h"
 #include "gtkradioaction.h"
 #include "gtkaccelmap.h"
 #include "gtkmarshalers.h"
+#include "gtkbuilderprivate.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkalias.h"
@@ -52,7 +56,7 @@ struct _GtkActionGroupPrivate
 
   GtkTranslateFunc translate_func;
   gpointer         translate_data;
-  GtkDestroyNotify translate_notify;   
+  GDestroyNotify   translate_notify;
 };
 
 enum 
@@ -86,6 +90,26 @@ static void       gtk_action_group_get_property    (GObject             *object,
 static GtkAction *gtk_action_group_real_get_action (GtkActionGroup      *self,
 						    const gchar         *name);
 
+/* GtkBuildable */
+static void gtk_action_group_buildable_init (GtkBuildableIface *iface);
+static void gtk_action_group_buildable_add_child (GtkBuildable  *buildable,
+						  GtkBuilder    *builder,
+						  GObject       *child,
+						  const gchar   *type);
+static void gtk_action_group_buildable_set_name (GtkBuildable *buildable,
+						 const gchar  *name);
+static const gchar* gtk_action_group_buildable_get_name (GtkBuildable *buildable);
+static gboolean gtk_action_group_buildable_custom_tag_start (GtkBuildable     *buildable,
+							     GtkBuilder       *builder,
+							     GObject          *child,
+							     const gchar      *tagname,
+							     GMarkupParser    *parser,
+							     gpointer         *data);
+static void gtk_action_group_buildable_custom_tag_end (GtkBuildable *buildable,
+						       GtkBuilder   *builder,
+						       GObject      *child,
+						       const gchar  *tagname,
+						       gpointer     *user_data);
 
 GType
 gtk_action_group_get_type (void)
@@ -94,7 +118,7 @@ gtk_action_group_get_type (void)
 
   if (!type)
     {
-      static const GTypeInfo type_info =
+      const GTypeInfo type_info =
       {
         sizeof (GtkActionGroupClass),
 	NULL,           /* base_init */
@@ -107,10 +131,20 @@ gtk_action_group_get_type (void)
         (GInstanceInitFunc) gtk_action_group_init,
       };
 
+      static const GInterfaceInfo buildable_info =
+      {
+	(GInterfaceInitFunc) gtk_action_group_buildable_init,
+	NULL,
+	NULL
+      };
+
       type = g_type_register_static (G_TYPE_OBJECT, I_("GtkActionGroup"),
 				     &type_info, 0);
-    }
 
+      g_type_add_interface_static (type,
+				   GTK_TYPE_BUILDABLE,
+				   &buildable_info);
+    }
   return type;
 }
 
@@ -158,7 +192,7 @@ gtk_action_group_class_init (GtkActionGroupClass *klass)
    * @action: the action
    * @proxy: the proxy
    *
-   * The connect_proxy signal is emitted after connecting a proxy to 
+   * The ::connect-proxy signal is emitted after connecting a proxy to 
    * an action in the group. Note that the proxy may have been connected 
    * to a different action before.
    *
@@ -173,7 +207,7 @@ gtk_action_group_class_init (GtkActionGroupClass *klass)
    * Since: 2.4
    */
   action_group_signals[CONNECT_PROXY] =
-    g_signal_new (I_("connect_proxy"),
+    g_signal_new (I_("connect-proxy"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  0, 0, NULL, NULL,
 		  _gtk_marshal_VOID__OBJECT_OBJECT,
@@ -186,7 +220,7 @@ gtk_action_group_class_init (GtkActionGroupClass *klass)
    * @action: the action
    * @proxy: the proxy
    *
-   * The disconnect_proxy signal is emitted after disconnecting a proxy 
+   * The ::disconnect-proxy signal is emitted after disconnecting a proxy 
    * from an action in the group. 
    *
    * #GtkUIManager proxies the signal and provides global notification 
@@ -196,7 +230,7 @@ gtk_action_group_class_init (GtkActionGroupClass *klass)
    * Since: 2.4
    */
   action_group_signals[DISCONNECT_PROXY] =
-    g_signal_new (I_("disconnect_proxy"),
+    g_signal_new (I_("disconnect-proxy"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  0, 0, NULL, NULL,
 		  _gtk_marshal_VOID__OBJECT_OBJECT,
@@ -208,7 +242,7 @@ gtk_action_group_class_init (GtkActionGroupClass *klass)
    * @action_group: the group
    * @action: the action
    *
-   * The pre_activate signal is emitted just before the @action in the
+   * The ::pre-activate signal is emitted just before the @action in the
    * @action_group is activated
    *
    * This is intended for #GtkUIManager to proxy the signal and provide global
@@ -217,7 +251,7 @@ gtk_action_group_class_init (GtkActionGroupClass *klass)
    * Since: 2.4
    */
   action_group_signals[PRE_ACTIVATE] =
-    g_signal_new (I_("pre_activate"),
+    g_signal_new (I_("pre-activate"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  0, 0, NULL, NULL,
 		  _gtk_marshal_VOID__OBJECT,
@@ -229,7 +263,7 @@ gtk_action_group_class_init (GtkActionGroupClass *klass)
    * @action_group: the group
    * @action: the action
    *
-   * The post_activate signal is emitted just after the @action in the
+   * The ::post-activate signal is emitted just after the @action in the
    * @action_group is activated
    *
    * This is intended for #GtkUIManager to proxy the signal and provide global
@@ -238,7 +272,7 @@ gtk_action_group_class_init (GtkActionGroupClass *klass)
    * Since: 2.4
    */
   action_group_signals[POST_ACTIVATE] =
-    g_signal_new (I_("post_activate"),
+    g_signal_new (I_("post-activate"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  0, 0, NULL, NULL,
 		  _gtk_marshal_VOID__OBJECT,
@@ -259,16 +293,166 @@ remove_action (GtkAction *action)
 static void
 gtk_action_group_init (GtkActionGroup *self)
 {
-  self->private_data = GTK_ACTION_GROUP_GET_PRIVATE (self);
-  self->private_data->name = NULL;
-  self->private_data->sensitive = TRUE;
-  self->private_data->visible = TRUE;
-  self->private_data->actions = g_hash_table_new_full (g_str_hash, g_str_equal,
-						       (GDestroyNotify) g_free,
-						       (GDestroyNotify) remove_action);
-  self->private_data->translate_func = NULL;
-  self->private_data->translate_data = NULL;
-  self->private_data->translate_notify = NULL;
+  GtkActionGroupPrivate *private;
+
+  private = GTK_ACTION_GROUP_GET_PRIVATE (self);
+
+  private->name = NULL;
+  private->sensitive = TRUE;
+  private->visible = TRUE;
+  private->actions = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                            NULL,
+                                            (GDestroyNotify) remove_action);
+  private->translate_func = NULL;
+  private->translate_data = NULL;
+  private->translate_notify = NULL;
+}
+
+static void
+gtk_action_group_buildable_init (GtkBuildableIface *iface)
+{
+  iface->add_child = gtk_action_group_buildable_add_child;
+  iface->set_name = gtk_action_group_buildable_set_name;
+  iface->get_name = gtk_action_group_buildable_get_name;
+  iface->custom_tag_start = gtk_action_group_buildable_custom_tag_start;
+  iface->custom_tag_end = gtk_action_group_buildable_custom_tag_end;
+}
+
+static void
+gtk_action_group_buildable_add_child (GtkBuildable  *buildable,
+				      GtkBuilder    *builder,
+				      GObject       *child,
+				      const gchar   *type)
+{
+  gtk_action_group_add_action_with_accel (GTK_ACTION_GROUP (buildable),
+					  GTK_ACTION (child), NULL);
+}
+
+static void
+gtk_action_group_buildable_set_name (GtkBuildable *buildable,
+				     const gchar  *name)
+{
+  GtkActionGroup *self = GTK_ACTION_GROUP (buildable);
+  GtkActionGroupPrivate *private = GTK_ACTION_GROUP_GET_PRIVATE (self);
+
+  private->name = g_strdup (name);
+}
+
+static const gchar *
+gtk_action_group_buildable_get_name (GtkBuildable *buildable)
+{
+  GtkActionGroup *self = GTK_ACTION_GROUP (buildable);
+  GtkActionGroupPrivate *private = GTK_ACTION_GROUP_GET_PRIVATE (self);
+  return private->name;
+}
+
+typedef struct {
+  GObject         *child;
+  guint            key;
+  GdkModifierType  modifiers;
+} AcceleratorParserData;
+
+static void
+accelerator_start_element (GMarkupParseContext *context,
+			   const gchar         *element_name,
+			   const gchar        **names,
+			   const gchar        **values,
+			   gpointer             user_data,
+			   GError             **error)
+{
+  gint i;
+  guint key = 0;
+  GdkModifierType modifiers = 0;
+  AcceleratorParserData *parser_data = (AcceleratorParserData*)user_data;
+
+  if (strcmp (element_name, "accelerator") != 0)
+    g_warning ("Unknown <accelerator> tag: %s", element_name);
+
+  for (i = 0; names[i]; i++)
+    {
+      if (strcmp (names[i], "key") == 0)
+	key = gdk_keyval_from_name (values[i]);
+      else if (strcmp (names[i], "modifiers") == 0)
+	{
+	  if (!_gtk_builder_flags_from_string (GDK_TYPE_MODIFIER_TYPE,
+					       values[i],
+					       &modifiers,
+					       error))
+	      return;
+	}
+    }
+
+  if (key == 0)
+    {
+      g_warning ("<accelerator> requires a key attribute");
+      return;
+    }
+  parser_data->key = key;
+  parser_data->modifiers = modifiers;
+}
+
+static const GMarkupParser accelerator_parser =
+  {
+    accelerator_start_element
+  };
+
+static gboolean
+gtk_action_group_buildable_custom_tag_start (GtkBuildable     *buildable,
+					     GtkBuilder       *builder,
+					     GObject          *child,
+					     const gchar      *tagname,
+					     GMarkupParser    *parser,
+					     gpointer         *user_data)
+{
+  AcceleratorParserData *parser_data;
+
+  if (child && strcmp (tagname, "accelerator") == 0)
+    {
+      parser_data = g_slice_new0 (AcceleratorParserData);
+      parser_data->child = child;
+      *user_data = parser_data;
+      *parser = accelerator_parser;
+
+      return TRUE;
+    }
+  return FALSE;
+}
+
+static void
+gtk_action_group_buildable_custom_tag_end (GtkBuildable *buildable,
+					   GtkBuilder   *builder,
+					   GObject      *child,
+					   const gchar  *tagname,
+					   gpointer     *user_data)
+{
+  AcceleratorParserData *data;
+  
+  if (strcmp (tagname, "accelerator") == 0)
+    {
+      GtkActionGroup *action_group;
+      GtkActionGroupPrivate *private;
+      GtkAction *action;
+      gchar *accel_path;
+      
+      data = (AcceleratorParserData*)user_data;
+      action_group = GTK_ACTION_GROUP (buildable);
+      private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
+      action = GTK_ACTION (child);
+	
+      accel_path = g_strconcat ("<Actions>/",
+				private->name, "/",
+				gtk_action_get_name (action), NULL);
+
+      if (gtk_accel_map_lookup_entry (accel_path, NULL))
+	gtk_accel_map_change_entry (accel_path, data->key, data->modifiers, TRUE);
+      else
+	gtk_accel_map_add_entry (accel_path, data->key, data->modifiers);
+
+      gtk_action_set_accel_path (action, accel_path);
+      
+      g_free (accel_path);
+      g_slice_free (AcceleratorParserData, data);
+    }
 }
 
 /**
@@ -287,9 +471,11 @@ GtkActionGroup *
 gtk_action_group_new (const gchar *name)
 {
   GtkActionGroup *self;
+  GtkActionGroupPrivate *private;
 
   self = g_object_new (GTK_TYPE_ACTION_GROUP, NULL);
-  self->private_data->name = g_strdup (name);
+  private = GTK_ACTION_GROUP_GET_PRIVATE (self);
+  private->name = g_strdup (name);
 
   return self;
 }
@@ -298,20 +484,21 @@ static void
 gtk_action_group_finalize (GObject *object)
 {
   GtkActionGroup *self;
+  GtkActionGroupPrivate *private;
 
   self = GTK_ACTION_GROUP (object);
+  private = GTK_ACTION_GROUP_GET_PRIVATE (self);
 
-  g_free (self->private_data->name);
-  self->private_data->name = NULL;
+  g_free (private->name);
+  private->name = NULL;
 
-  g_hash_table_destroy (self->private_data->actions);
-  self->private_data->actions = NULL;
+  g_hash_table_destroy (private->actions);
+  private->actions = NULL;
 
-  if (self->private_data->translate_notify)
-    self->private_data->translate_notify (self->private_data->translate_data);
+  if (private->translate_notify)
+    private->translate_notify (private->translate_data);
 
-  if (parent_class->finalize)
-    (* parent_class->finalize) (object);
+  parent_class->finalize (object);
 }
 
 static void
@@ -321,15 +508,17 @@ gtk_action_group_set_property (GObject         *object,
 			       GParamSpec      *pspec)
 {
   GtkActionGroup *self;
+  GtkActionGroupPrivate *private;
   gchar *tmp;
   
   self = GTK_ACTION_GROUP (object);
+  private = GTK_ACTION_GROUP_GET_PRIVATE (self);
 
   switch (prop_id)
     {
     case PROP_NAME:
-      tmp = self->private_data->name;
-      self->private_data->name = g_value_dup_string (value);
+      tmp = private->name;
+      private->name = g_value_dup_string (value);
       g_free (tmp);
       break;
     case PROP_SENSITIVE:
@@ -351,19 +540,21 @@ gtk_action_group_get_property (GObject    *object,
 			       GParamSpec *pspec)
 {
   GtkActionGroup *self;
+  GtkActionGroupPrivate *private;
   
   self = GTK_ACTION_GROUP (object);
+  private = GTK_ACTION_GROUP_GET_PRIVATE (self);
 
   switch (prop_id)
     {
     case PROP_NAME:
-      g_value_set_string (value, self->private_data->name);
+      g_value_set_string (value, private->name);
       break;
     case PROP_SENSITIVE:
-      g_value_set_boolean (value, self->private_data->sensitive);
+      g_value_set_boolean (value, private->sensitive);
       break;
     case PROP_VISIBLE:
-      g_value_set_boolean (value, self->private_data->visible);
+      g_value_set_boolean (value, private->visible);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -375,7 +566,11 @@ static GtkAction *
 gtk_action_group_real_get_action (GtkActionGroup *self,
 				  const gchar    *action_name)
 {
-  return g_hash_table_lookup (self->private_data->actions, action_name);
+  GtkActionGroupPrivate *private;
+
+  private = GTK_ACTION_GROUP_GET_PRIVATE (self);
+
+  return g_hash_table_lookup (private->actions, action_name);
 }
 
 /**
@@ -391,9 +586,13 @@ gtk_action_group_real_get_action (GtkActionGroup *self,
 G_CONST_RETURN gchar *
 gtk_action_group_get_name (GtkActionGroup *action_group)
 {
+  GtkActionGroupPrivate *private;
+
   g_return_val_if_fail (GTK_IS_ACTION_GROUP (action_group), NULL);
 
-  return action_group->private_data->name;
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
+
+  return private->name;
 }
 
 /**
@@ -412,9 +611,13 @@ gtk_action_group_get_name (GtkActionGroup *action_group)
 gboolean
 gtk_action_group_get_sensitive (GtkActionGroup *action_group)
 {
+  GtkActionGroupPrivate *private;
+
   g_return_val_if_fail (GTK_IS_ACTION_GROUP (action_group), FALSE);
 
-  return action_group->private_data->sensitive;
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
+
+  return private->sensitive;
 }
 
 static void
@@ -440,14 +643,17 @@ void
 gtk_action_group_set_sensitive (GtkActionGroup *action_group, 
 				gboolean        sensitive)
 {
+  GtkActionGroupPrivate *private;
+
   g_return_if_fail (GTK_IS_ACTION_GROUP (action_group));
 
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
   sensitive = sensitive != FALSE;
 
-  if (action_group->private_data->sensitive != sensitive)
+  if (private->sensitive != sensitive)
     {
-      action_group->private_data->sensitive = sensitive;
-      g_hash_table_foreach (action_group->private_data->actions, 
+      private->sensitive = sensitive;
+      g_hash_table_foreach (private->actions, 
 			    (GHFunc) cb_set_action_sensitivity, NULL);
 
       g_object_notify (G_OBJECT (action_group), "sensitive");
@@ -470,9 +676,13 @@ gtk_action_group_set_sensitive (GtkActionGroup *action_group,
 gboolean
 gtk_action_group_get_visible (GtkActionGroup *action_group)
 {
+  GtkActionGroupPrivate *private;
+
   g_return_val_if_fail (GTK_IS_ACTION_GROUP (action_group), FALSE);
 
-  return action_group->private_data->visible;
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
+
+  return private->visible;
 }
 
 static void
@@ -498,14 +708,17 @@ void
 gtk_action_group_set_visible (GtkActionGroup *action_group, 
 			      gboolean        visible)
 {
+  GtkActionGroupPrivate *private;
+
   g_return_if_fail (GTK_IS_ACTION_GROUP (action_group));
 
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
   visible = visible != FALSE;
 
-  if (action_group->private_data->visible != visible)
+  if (private->visible != visible)
     {
-      action_group->private_data->visible = visible;
-      g_hash_table_foreach (action_group->private_data->actions, 
+      private->visible = visible;
+      g_hash_table_foreach (private->actions, 
 			    (GHFunc) cb_set_action_visiblity, NULL);
 
       g_object_notify (G_OBJECT (action_group), "visible");
@@ -530,8 +743,27 @@ gtk_action_group_get_action (GtkActionGroup *action_group,
   g_return_val_if_fail (GTK_IS_ACTION_GROUP (action_group), NULL);
   g_return_val_if_fail (GTK_ACTION_GROUP_GET_CLASS (action_group)->get_action != NULL, NULL);
 
-  return (* GTK_ACTION_GROUP_GET_CLASS (action_group)->get_action)
-    (action_group, action_name);
+  return GTK_ACTION_GROUP_GET_CLASS (action_group)->get_action (action_group,
+                                                                action_name);
+}
+
+static gboolean
+check_unique_action (GtkActionGroup *action_group,
+	             const gchar    *action_name)
+{
+  if (gtk_action_group_get_action (action_group, action_name) != NULL)
+    {
+      GtkActionGroupPrivate *private;
+
+      private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
+
+      g_warning ("Refusing to add non-unique action '%s' to action group '%s'",
+	 	 action_name,
+		 private->name);
+      return FALSE;
+    }
+
+  return TRUE;
 }
 
 /**
@@ -552,12 +784,22 @@ void
 gtk_action_group_add_action (GtkActionGroup *action_group,
 			     GtkAction      *action)
 {
+  GtkActionGroupPrivate *private;
+  const gchar *name;
+
   g_return_if_fail (GTK_IS_ACTION_GROUP (action_group));
   g_return_if_fail (GTK_IS_ACTION (action));
-  g_return_if_fail (gtk_action_get_name (action) != NULL);
 
-  g_hash_table_insert (action_group->private_data->actions, 
-		       g_strdup (gtk_action_get_name (action)),
+  name = gtk_action_get_name (action);
+  g_return_if_fail (name != NULL);
+  
+  if (!check_unique_action (action_group, name))
+    return;
+
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
+
+  g_hash_table_insert (private->actions, 
+		       (gpointer) name,
                        g_object_ref (action));
   g_object_set (action, I_("action-group"), action_group, NULL);
 }
@@ -585,17 +827,19 @@ gtk_action_group_add_action_with_accel (GtkActionGroup *action_group,
 					GtkAction      *action,
 					const gchar    *accelerator)
 {
+  GtkActionGroupPrivate *private;
   gchar *accel_path;
   guint  accel_key = 0;
   GdkModifierType accel_mods;
-  GtkStockItem stock_item;
-  gchar *name;
-  gchar *stock_id;
-  
-  g_object_get (action, "name", &name, "stock-id", &stock_id, NULL);
+  const gchar *name;
 
+  name = gtk_action_get_name (action);
+  if (!check_unique_action (action_group, name))
+    return;
+
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
   accel_path = g_strconcat ("<Actions>/",
-			    action_group->private_data->name, "/", name, NULL);
+			    private->name, "/", name, NULL);
 
   if (accelerator)
     {
@@ -609,10 +853,20 @@ gtk_action_group_add_action_with_accel (GtkActionGroup *action_group,
 		       accelerator, name);
 	}
     }
-  else if (stock_id && gtk_stock_lookup (stock_id, &stock_item))
+  else 
     {
-      accel_key = stock_item.keyval;
-      accel_mods = stock_item.modifier;
+      gchar *stock_id;
+      GtkStockItem stock_item;
+
+      g_object_get (action, "stock-id", &stock_id, NULL);
+
+      if (stock_id && gtk_stock_lookup (stock_id, &stock_item))
+        {
+          accel_key = stock_item.keyval;
+          accel_mods = stock_item.modifier;
+	}
+
+      g_free (stock_id);
     }
 
   if (accel_key)
@@ -622,8 +876,6 @@ gtk_action_group_add_action_with_accel (GtkActionGroup *action_group,
   gtk_action_group_add_action (action_group, action);
 
   g_free (accel_path);
-  g_free (stock_id);
-  g_free (name);
 }
 
 /**
@@ -639,14 +891,18 @@ void
 gtk_action_group_remove_action (GtkActionGroup *action_group,
 				GtkAction      *action)
 {
+  GtkActionGroupPrivate *private;
+  const gchar *name;
+
   g_return_if_fail (GTK_IS_ACTION_GROUP (action_group));
   g_return_if_fail (GTK_IS_ACTION (action));
-  g_return_if_fail (gtk_action_get_name (action) != NULL);
 
-  /* extra protection to make sure action->name is valid */
-  g_object_ref (action);
-  g_hash_table_remove (action_group->private_data->actions, gtk_action_get_name (action));
-  g_object_unref (action);
+  name = gtk_action_get_name (action);
+  g_return_if_fail (name != NULL);
+
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
+
+  g_hash_table_remove (private->actions, name);
 }
 
 static void
@@ -672,10 +928,14 @@ add_single_action (gpointer key,
 GList *
 gtk_action_group_list_actions (GtkActionGroup *action_group)
 {
+  GtkActionGroupPrivate *private;
   GList *actions = NULL;
+
   g_return_val_if_fail (GTK_IS_ACTION_GROUP (action_group), NULL);
+
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
   
-  g_hash_table_foreach (action_group->private_data->actions, add_single_action, &actions);
+  g_hash_table_foreach (private->actions, add_single_action, &actions);
 
   return g_list_reverse (actions);
 }
@@ -724,9 +984,9 @@ shared_data_unref (gpointer data)
   shared_data->ref_count--;
   if (shared_data->ref_count == 0)
     {
-      if (shared_data->destroy) 
-	(*shared_data->destroy) (shared_data->data);
-      
+      if (shared_data->destroy)
+	shared_data->destroy (shared_data->data);
+
       g_slice_free (SharedData, shared_data);
     }
 }
@@ -772,6 +1032,9 @@ gtk_action_group_add_actions_full (GtkActionGroup       *action_group,
       const gchar *label;
       const gchar *tooltip;
 
+      if (!check_unique_action (action_group, entries[i].name))
+        continue;
+
       label = gtk_action_group_translate_string (action_group, entries[i].label);
       tooltip = gtk_action_group_translate_string (action_group, entries[i].tooltip);
 
@@ -782,9 +1045,9 @@ gtk_action_group_add_actions_full (GtkActionGroup       *action_group,
 
       if (entries[i].stock_id) 
 	{
-	  if (gtk_icon_factory_lookup_default (entries[i].stock_id))
-	    g_object_set (action, "stock-id", entries[i].stock_id, NULL);
-	  else
+	  g_object_set (action, "stock-id", entries[i].stock_id, NULL);
+	  if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (), 
+				       entries[i].stock_id))
 	    g_object_set (action, "icon-name", entries[i].stock_id, NULL);
 	}
 	  
@@ -876,6 +1139,9 @@ gtk_action_group_add_toggle_actions_full (GtkActionGroup             *action_gro
       const gchar *label;
       const gchar *tooltip;
 
+      if (!check_unique_action (action_group, entries[i].name))
+        continue;
+
       label = gtk_action_group_translate_string (action_group, entries[i].label);
       tooltip = gtk_action_group_translate_string (action_group, entries[i].tooltip);
 
@@ -912,7 +1178,7 @@ gtk_action_group_add_toggle_actions_full (GtkActionGroup             *action_gro
       g_object_unref (action);
     }
 
-    shared_data_unref (shared_data);
+  shared_data_unref (shared_data);
 }
 
 /**
@@ -988,6 +1254,9 @@ gtk_action_group_add_radio_actions_full (GtkActionGroup            *action_group
       const gchar *label;
       const gchar *tooltip; 
 
+      if (!check_unique_action (action_group, entries[i].name))
+        continue;
+
       label = gtk_action_group_translate_string (action_group, entries[i].label);
       tooltip = gtk_action_group_translate_string (action_group, entries[i].tooltip);
 
@@ -1031,9 +1300,9 @@ gtk_action_group_add_radio_actions_full (GtkActionGroup            *action_group
  * @action_group: a #GtkActionGroup
  * @func: a #GtkTranslateFunc
  * @data: data to be passed to @func and @notify
- * @notify: a #GtkDestroyNotify function to be called when @action_group is 
+ * @notify: a #GDestroyNotify function to be called when @action_group is
  *   destroyed and when the translation function is changed again
- * 
+ *
  * Sets a function to be used for translating the @label and @tooltip of 
  * #GtkActionGroupEntry<!-- -->s added by gtk_action_group_add_actions().
  *
@@ -1046,25 +1315,29 @@ void
 gtk_action_group_set_translate_func (GtkActionGroup   *action_group,
 				     GtkTranslateFunc  func,
 				     gpointer          data,
-				     GtkDestroyNotify  notify)
+				     GDestroyNotify    notify)
 {
+  GtkActionGroupPrivate *private;
+
   g_return_if_fail (GTK_IS_ACTION_GROUP (action_group));
   
-  if (action_group->private_data->translate_notify)
-    action_group->private_data->translate_notify (action_group->private_data->translate_data);
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
+
+  if (private->translate_notify)
+    private->translate_notify (private->translate_data);
       
-  action_group->private_data->translate_func = func;
-  action_group->private_data->translate_data = data;
-  action_group->private_data->translate_notify = notify;
+  private->translate_func = func;
+  private->translate_data = data;
+  private->translate_notify = notify;
 }
 
 static gchar *
 dgettext_swapped (const gchar *msgid, 
 		  const gchar *domainname)
 {
-  /* Pass through dgettext if and only if msgid is nonempty. */
+  /* Pass through g_dgettext if and only if msgid is nonempty. */
   if (msgid && *msgid) 
-    return dgettext (domainname, msgid); 
+    return (gchar*) g_dgettext (domainname, msgid); 
   else
     return (gchar*) msgid;
 }
@@ -1072,9 +1345,9 @@ dgettext_swapped (const gchar *msgid,
 /**
  * gtk_action_group_set_translation_domain:
  * @action_group: a #GtkActionGroup
- * @domain: the translation domain to use for dgettext() calls
+ * @domain: the translation domain to use for g_dgettext() calls
  * 
- * Sets the translation domain and uses dgettext() for translating the 
+ * Sets the translation domain and uses g_dgettext() for translating the 
  * @label and @tooltip of #GtkActionEntry<!-- -->s added by 
  * gtk_action_group_add_actions().
  *
@@ -1112,6 +1385,7 @@ G_CONST_RETURN gchar *
 gtk_action_group_translate_string (GtkActionGroup *action_group,
 				   const gchar    *string)
 {
+  GtkActionGroupPrivate *private;
   GtkTranslateFunc translate_func;
   gpointer translate_data;
   
@@ -1120,8 +1394,10 @@ gtk_action_group_translate_string (GtkActionGroup *action_group,
   if (string == NULL)
     return NULL;
 
-  translate_func = action_group->private_data->translate_func;
-  translate_data = action_group->private_data->translate_data;
+  private = GTK_ACTION_GROUP_GET_PRIVATE (action_group);
+
+  translate_func = private->translate_func;
+  translate_data = private->translate_data;
   
   if (translate_func)
     return translate_func (string, translate_data);
