@@ -21,9 +21,13 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_DIALOG_H__
 #define __GTK_DIALOG_H__
 
@@ -32,9 +36,7 @@
 #include <gtk/gtkwindow.h>
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 /* Parameters for dialog construction */
 typedef enum
@@ -83,11 +85,11 @@ typedef enum
 
 
 #define GTK_TYPE_DIALOG                  (gtk_dialog_get_type ())
-#define GTK_DIALOG(obj)                  (GTK_CHECK_CAST ((obj), GTK_TYPE_DIALOG, GtkDialog))
-#define GTK_DIALOG_CLASS(klass)          (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_DIALOG, GtkDialogClass))
-#define GTK_IS_DIALOG(obj)               (GTK_CHECK_TYPE ((obj), GTK_TYPE_DIALOG))
-#define GTK_IS_DIALOG_CLASS(klass)       (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_DIALOG))
-#define GTK_DIALOG_GET_CLASS(obj)        (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_DIALOG, GtkDialogClass))
+#define GTK_DIALOG(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_DIALOG, GtkDialog))
+#define GTK_DIALOG_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_DIALOG, GtkDialogClass))
+#define GTK_IS_DIALOG(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_DIALOG))
+#define GTK_IS_DIALOG_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_DIALOG))
+#define GTK_DIALOG_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_DIALOG, GtkDialogClass))
 
 
 typedef struct _GtkDialog        GtkDialog;
@@ -98,11 +100,11 @@ struct _GtkDialog
   GtkWindow window;
 
   /*< public >*/
-  GtkWidget *vbox;
-  GtkWidget *action_area;
+  GtkWidget *GSEAL (vbox);
+  GtkWidget *GSEAL (action_area);
 
   /*< private >*/
-  GtkWidget *separator;
+  GtkWidget *GSEAL (separator);
 };
 
 struct _GtkDialogClass
@@ -123,7 +125,7 @@ struct _GtkDialogClass
 };
 
 
-GtkType    gtk_dialog_get_type (void) G_GNUC_CONST;
+GType      gtk_dialog_get_type (void) G_GNUC_CONST;
 GtkWidget* gtk_dialog_new      (void);
 
 GtkWidget* gtk_dialog_new_with_buttons (const gchar     *title,
@@ -140,18 +142,28 @@ GtkWidget* gtk_dialog_add_button        (GtkDialog   *dialog,
                                          gint         response_id);
 void       gtk_dialog_add_buttons       (GtkDialog   *dialog,
                                          const gchar *first_button_text,
-                                         ...);
+                                         ...) G_GNUC_NULL_TERMINATED;
 
 void gtk_dialog_set_response_sensitive (GtkDialog *dialog,
                                         gint       response_id,
                                         gboolean   setting);
 void gtk_dialog_set_default_response   (GtkDialog *dialog,
                                         gint       response_id);
+gint gtk_dialog_get_response_for_widget (GtkDialog *dialog,
+					 GtkWidget *widget);
 
 void     gtk_dialog_set_has_separator (GtkDialog *dialog,
                                        gboolean   setting);
 gboolean gtk_dialog_get_has_separator (GtkDialog *dialog);
 
+gboolean gtk_alternative_dialog_button_order (GdkScreen *screen);
+void     gtk_dialog_set_alternative_button_order (GtkDialog *dialog,
+						  gint       first_response_id,
+						  ...);
+void     gtk_dialog_set_alternative_button_order_from_array (GtkDialog *dialog,
+                                                             gint       n_params,
+                                                             gint      *new_order);
+
 /* Emit response signal */
 void gtk_dialog_response           (GtkDialog *dialog,
                                     gint       response_id);
@@ -159,9 +171,13 @@ void gtk_dialog_response           (GtkDialog *dialog,
 /* Returns response_id */
 gint gtk_dialog_run                (GtkDialog *dialog);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+GtkWidget * gtk_dialog_get_action_area  (GtkDialog *dialog);
+GtkWidget * gtk_dialog_get_content_area (GtkDialog *dialog);
 
+/* For private use only */
+void _gtk_dialog_set_ignore_separator (GtkDialog *dialog,
+				       gboolean   ignore_separator);
+
+G_END_DECLS
 
 #endif /* __GTK_DIALOG_H__ */
