@@ -416,6 +416,14 @@ gtk_recent_manager_real_changed (GtkRecentManager *manager)
 	  g_error_free (write_error);
 	}
 
+      if (g_chmod (priv->filename, 0600) < 0)
+        {
+          filename_warning ("Attempting to set the permissions of `%s', "
+                            "but failed: %s",
+                            priv->filename,
+                            g_strerror (errno));
+        }
+
       /* mark us as clean */
       priv->is_dirty = FALSE;
     }
@@ -1018,7 +1026,12 @@ gtk_recent_manager_remove_item (GtkRecentManager  *manager,
   g_bookmark_file_remove_item (priv->recent_items, uri, &remove_error);
   if (remove_error)
     {
-      g_propagate_error (error, remove_error);
+      g_error_free (remove_error);
+
+      g_set_error (error, GTK_RECENT_MANAGER_ERROR,
+		   GTK_RECENT_MANAGER_ERROR_NOT_FOUND,
+		   _("Unable to find an item with URI '%s'"),
+		   uri);
       
       return FALSE;
     }
@@ -1214,6 +1227,15 @@ gtk_recent_manager_move_item (GtkRecentManager  *recent_manager,
   
   priv = recent_manager->priv;
 
+  if (!priv->recent_items)
+    {
+      g_set_error (error, GTK_RECENT_MANAGER_ERROR,
+      		   GTK_RECENT_MANAGER_ERROR_NOT_FOUND,
+      		   _("Unable to find an item with URI '%s'"),
+      		   uri);
+      return FALSE;
+    }
+
   if (!g_bookmark_file_has_item (priv->recent_items, uri))
     {
       g_set_error (error, GTK_RECENT_MANAGER_ERROR,
@@ -1229,7 +1251,12 @@ gtk_recent_manager_move_item (GtkRecentManager  *recent_manager,
                                    &move_error);
   if (move_error)
     {
-      g_propagate_error (error, move_error);
+      g_error_free (move_error);
+
+      g_set_error (error, GTK_RECENT_MANAGER_ERROR,
+      		   GTK_RECENT_MANAGER_ERROR_NOT_FOUND,
+      		   _("Unable to find an item with URI '%s'"),
+      		   uri);
       return FALSE;
     }
   
@@ -1671,11 +1698,11 @@ recent_app_info_new (const gchar *app_name)
 
   g_assert (app_name != NULL);
   
-  app_info = g_new0 (RecentAppInfo, 1);
+  app_info = g_slice_new0 (RecentAppInfo);
   app_info->name = g_strdup (app_name);
   app_info->exec = NULL;
   app_info->count = 1;
-  app_info->stamp = time (NULL);
+  app_info->stamp = 0; 
   
   return app_info;
 }
@@ -1687,10 +1714,9 @@ recent_app_info_free (RecentAppInfo *app_info)
     return;
   
   g_free (app_info->name);
-  
   g_free (app_info->exec);
   
-  g_free (app_info);
+  g_slice_free (RecentAppInfo, app_info);
 }
 
 /**
@@ -1862,52 +1888,31 @@ get_icon_for_mime_type (const char *mime_type,
 			gint        pixel_size)
 {
   GtkIconTheme *icon_theme;
-  const char *separator;
-  GString *icon_name;
+  char *content_type;
+  GIcon *icon;
+  GtkIconInfo *info;
   GdkPixbuf *pixbuf;
 
-  separator = strchr (mime_type, '/');
-  if (!separator)
-    return NULL;
-
   icon_theme = gtk_icon_theme_get_default ();
 
-  /* try with the three icon name variants for MIME types */
+  content_type = g_content_type_from_mime_type (mime_type);
 
-  /* canonicalize MIME type: foo/x-bar -> foo-x-bar */
-  icon_name = g_string_new (NULL);
-  g_string_append_len (icon_name, mime_type, separator - mime_type);
-  g_string_append_c (icon_name, '-');
-  g_string_append (icon_name, separator + 1);
-  pixbuf = gtk_icon_theme_load_icon (icon_theme, icon_name->str,
-                                     pixel_size,
-                                     0,
-                                     NULL);
-  g_string_free (icon_name, TRUE);
-  if (pixbuf)
-    return pixbuf;
+  if (!content_type)
+    return NULL;
 
-  /* canonicalize MIME type, and prepend "gnome-mime-" */
-  icon_name = g_string_new ("gnome-mime-");
-  g_string_append_len (icon_name, mime_type, separator - mime_type);
-  g_string_append_c (icon_name, '-');
-  g_string_append (icon_name, separator + 1);
-  pixbuf = gtk_icon_theme_load_icon (icon_theme, icon_name->str,
-                                     pixel_size,
-                                     0,
-                                     NULL);
-  g_string_free (icon_name, TRUE);
-  if (pixbuf)
-    return pixbuf;
+  icon = g_content_type_get_icon (content_type);
+  info = gtk_icon_theme_lookup_by_gicon (icon_theme, 
+                                         icon, 
+                                         pixel_size, 
+                                         GTK_ICON_LOOKUP_USE_BUILTIN);
+  g_free (content_type);
+  g_object_unref (icon);
 
-  /* try the MIME family icon */
-  icon_name = g_string_new ("gnome-mime-");
-  g_string_append_len (icon_name, mime_type, separator - mime_type);
-  pixbuf = gtk_icon_theme_load_icon (icon_theme, icon_name->str,
-                                     pixel_size,
-                                     0,
-                                     NULL);
-  g_string_free (icon_name, TRUE);
+  if (!info)
+    return NULL;
+
+  pixbuf = gtk_icon_info_load_icon (info, NULL);
+  gtk_icon_info_free (info);
 
   return pixbuf;
 }
