@@ -407,8 +407,7 @@ gtk_ruler_unrealize (GtkWidget *widget)
       ruler->non_gr_exp_gc = NULL;
     }
 
-  if (GTK_WIDGET_CLASS (gtk_ruler_parent_class)->unrealize)
-    (* GTK_WIDGET_CLASS (gtk_ruler_parent_class)->unrealize) (widget);
+  GTK_WIDGET_CLASS (gtk_ruler_parent_class)->unrealize (widget);
 }
 
 static void
