@@ -121,7 +121,7 @@ gulong	gtk_signal_connect_full			(GtkObject	    *object,
 						 GtkSignalFunc	     func,
 						 GtkCallbackMarshal  unsupported,
 						 gpointer	     data,
-						 GtkDestroyNotify    destroy_func,
+						 GDestroyNotify      destroy_func,
 						 gint		     object_signal,
 						 gint		     after);
 void	gtk_signal_emitv			(GtkObject	    *object,
